-- indicators.sql
	
    

create or replace function inline_0 ()
returns integer as $body$
DECLARE
	v_id			integer;
BEGIN
	v_id := im_indicator__new(
		null, 'im_indicator', now(), 0, '', null,
		'Financial Documents per Project',
		'findocs_per_project',
		15110,
		15000,
		'select round(avg(cnt)) from (
select  count(*) as cnt,
        parent.project_id
from    im_costs c,
        im_projects parent,
        im_projects p
where   parent.parent_id is null and
        parent.end_date between now()::date-60 and now()::date-30 and
        p.tree_sortkey between parent.tree_sortkey and tree_right(parent.tree_sortkey) and
        c.project_id = p.project_id and
        c.cost_type_id not in (3716, 3718, 3720, 3714)
group by parent.project_id) t
;
',
		0,
		5,
		5
	);

	update im_indicators set
		indicator_section_id = 15245
	where indicator_id = v_id;

	update im_reports set
		report_description = 'Counts how many financial documents ' ||
		'(quotes, pos, invoices, bills, deliver notes or expense invoices) ' ||
		'have been generated for all main projects that ended (end_date) 30 days ago to 60 days ago.'
	where report_id = v_id;

        return 0;
end;$body$ language 'plpgsql';
select inline_0 ();
drop function inline_0 ();
    


create or replace function inline_0 ()
returns integer as $body$
DECLARE
	v_id			integer;
BEGIN
	v_id := im_indicator__new(
		null, 'im_indicator', now(), 0, '', null,
		'Revenues',
		'revenues',
		15110,
		15000,
		'select coalesce(round(sum(amount)), 0) from im_costs where cost_type_id in (select * from im_sub_categories(3700)) and effective_date between now()::date-60 and now()::date-30',
		0,
		1000,
		5
	);

	update im_indicators set
		indicator_section_id = 15200
	where indicator_id = v_id;

	update im_reports set
		report_description = 'Invoices written with effective date between 30 days and 60 days ago.'
	where report_id = v_id;

        return 0;
end;$body$ language 'plpgsql';
select inline_0 ();
drop function inline_0 ();

    

create or replace function inline_0 ()
returns integer as $body$
DECLARE
	v_id			integer;
BEGIN
	v_id := im_indicator__new(
		null, 'im_indicator', now(), 0, '', null,
		'External Costs',
		'external_costs',
		15110,
		15000,
		'select coalesce(round(sum(amount),0),0) from im_costs where cost_type_id = 3704 and effective_date between now()::date-60 and now()::date-30',
		0,
		1000,
		5
	);

	update im_indicators set
		indicator_section_id = 15200
	where indicator_id = v_id;

	update im_reports set
		report_description = 'Provider bills with effective date between 30 days ago and 60 days ago.'
	where report_id = v_id;

        return 0;
end;$body$ language 'plpgsql';
select inline_0 ();
drop function inline_0 ();
    


create or replace function inline_0 ()
returns integer as $body$
DECLARE
	v_id			integer;
BEGIN
	v_id := im_indicator__new(
		null, 'im_indicator', now(), 0, '', null,
		'Net Margin Two Months Ago',
		'net_margin',
		15110,
		15000,
		'select 1',
		0,
		20,
		5
	);

	update im_reports
	set report_sql = 'select
        round((invoices - bills - timesheet - expenses) / (invoices+0.000001) * 100, 1) as net_margin
from
        (select
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3700)) and effective_date between now()::date-60 and now()::date-30) as invoices,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3702)) and effective_date between now()::date-60 and now()::date-30) as quotes,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3704)) and effective_date between now()::date-60 and now()::date-30) as bills,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3706)) and effective_date between now()::date-60 and now()::date-30) as pos,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3718)) and effective_date between now()::date-60 and now()::date-30) as timesheet,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3720)) and effective_date between now()::date-60 and now()::date-30) as expenses
        ) base;'
	where report_name = 'Net Margin Two Months Ago';

	update im_indicators set
		indicator_section_id = 15200
	where indicator_id = v_id;

	update im_reports set
		report_description = 'Calculates ((Invoices - Bills - Timesheet - Expenses) / Invoices) of all financial documents between now-30 days and now-60 days.'
	where report_id = v_id;

        return 0;
end;$body$ language 'plpgsql';
select inline_0 ();
drop function inline_0 ();
    


create or replace function inline_0 ()
returns integer as $body$
DECLARE
	v_id			integer;
BEGIN
	v_id := im_indicator__new(
		null,
		'im_indicator',
		now(),
		0,
		'',
		null,
		'Preliminary Bruto Margin Two Months Ago',
		'prelim_bruto_margin',
		15110,
		15000,
		'select 1',
		0,
		60,
		5
	);

	update im_reports
	set report_sql = 'select
        round((quotes - pos) / (quotes+0.000001) * 100,1) as prelim_brut_margin
from
        (select
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3700)) and effective_date between now()::date-60 and now()::date-30) as invoices,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3702)) and effective_date between now()::date-60 and now()::date-30) as quotes,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3704)) and effective_date between now()::date-60 and now()::date-30) as bills,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3706)) and effective_date between now()::date-60 and now()::date-30) as pos,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3718)) and effective_date between now()::date-60 and now()::date-30) as timesheet,
        (select sum(amount) from im_costs where cost_type_id in (select * from im_sub_categories(3720)) and effective_date between now()::date-60 and now()::date-30) as expenses
        ) base;'
	where report_name = 'Preliminary Bruto Margin Two Months Ago';

	update im_indicators set
		indicator_section_id = 15200
	where indicator_id = v_id;

	update im_reports set
		report_description = 'Calculates ((Quotes - POs) / Quotes) of all financial documents effective between now-30 and now-60 (the 2nd last month).'
	where report_id = v_id;

        return 0;
end;$body$ language 'plpgsql';
select inline_0 ();
drop function inline_0 ();
    





-- Indicator component for the Finance Home Page
SELECT im_component_plugin__new (
	null,					-- plugin_id
	'im_component_plugin',			-- object_type
	now(),					-- creation_date
	null,					-- creation_user
	null,					-- creation_ip
	null,					-- context_id
	'Financial Indicators Timeline',	-- plugin_name - shown in menu
	'intranet-reporting-indicators',	-- package_name
	'right',				-- location
	'/intranet-cost/index',			-- page_url
	null,					-- view_name
	30,					-- sort_order
	'im_indicator_timeline_component -indicator_section_id [im_indicator_section_finance]',
	'lang::message::lookup "" intranet-cost.Financial_Indicators_Timeline "Financial Indicators Timeline"'
);





-- Update low and high level watermarks

update im_indicators set
	indicator_low_critical=10000, indicator_low_warn=20000, indicator_high_warn=null, indicator_high_critical=null
where indicator_id in (select report_id from im_reports where report_code = 'revenues');

